/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SELOR - Servidor de Emails Livre Orientado a Regras                           
   Programado por Lucas Priori                                                   
   Versao: 1.00                                                                  
                                                                                 
   Linux - make linux                                                            
   FreeBSD - gmake freebsd                                                       
                                                                                 
  *******************************************************************            
  Selor_mod_test e´ utilitario de testes para os modulos do servidor             
  Copyright (C) 2012,2013,2014,2015  Lucas Priori                                
                                                                                 
  This program is free software; you can redistribute it and/or                  
  modify it under the terms of the GNU General Public License                    
  as published by the Free Software Foundation; either version 2                 
  of the License, or (at your option) any later version.                         
                                                                                 
  This program is distributed in the hope that it will be useful,                
  but WITHOUT ANY WARRANTY; without even the implied warranty of                 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  
  GNU General Public License for more details.                                   
                                                                                 
  You should have received a copy of the GNU General Public License              
  along with this program; if not, write to the Free Software                    
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

#include<stdio.h>
#include<dlfcn.h>
#include<string.h>

#define TAM_mod_name 50
#define TAM_cmd 500
#define MOD_user_ncampos 4
#define TAM_dominio 100
#define NUM_itens_buffer 10
#define TAM_itens_buffer 150



//---------------------------------------------------------------------
char (*mod_init)(char *param, char *module_name, char *module_version, char *info, int info_buf_size);
//---------------------------------------------------------------------
char (*mod_dom_local)(char *dominio, char *ret_dom);
//---------------------------------------------------------------------
char (*mod_user_local)(char *email, char *info[], int num_campos);
//---------------------------------------------------------------------
int (*mod_ret_alias)(char *email, char *buffer_ret, int num_itens, int tam_item_buffer, unsigned int *prox_bloco);
//---------------------------------------------------------------------
char (*mod_auth)(char *email, char *senha, char *info[], int num_campos);
//---------------------------------------------------------------------
char (*mod_test_quota)(char *email, char *info[], int num_campos);
//---------------------------------------------------------------------
char *(*mod_shutdown)(void);
//---------------------------------------------------------------------
char *(*mod_msg_error)(void);
//---------------------------------------------------------------------




// -------------------------------------------------------------
//                     Programa principal                       
// -------------------------------------------------------------
int main(int argc, char *argv[]){
 void *modulo;
 char buffer_ret[NUM_itens_buffer][TAM_itens_buffer];
 char mod_id[TAM_mod_name + 1];
 char mod_version[200];
 char adomain[TAM_dominio];
 char mod_info_buffer[10];
 char cmd[TAM_cmd + 1];
 char *ptr_aux1;
 char *ptr_aux2;
 char *ptr_param=NULL;
 char ret1;
 char byte=0;
 char *mod_user_info[MOD_user_ncampos];    //Ponteiros temporarios com as informacoes extras dos usuarios
       // 0 = user           
       // 1 = password       
       // 2 = home directory 
       // 3 = dlocal         
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int mod_prox_bloco=0;
 
 if(argc < 2){
  printf("You need to specify the module path:\n");
  printf("./selor_mod_test /usr/local/selor/mod/selor_xxxx /usr/local/selor/mod/selor_xxxx.conf\n\n");
  return 0;
 }
 
 if(!(modulo=dlopen(argv[1], RTLD_NOW | RTLD_LOCAL))){
  printf("Dlerror: %s\n", dlerror());
  return 1;
 }
 
 if(argc > 2){
  // -- Tem parametro --
  ptr_param=argv[2];
 }
 
 mod_init=dlsym(modulo, "mod_init");
 mod_dom_local=dlsym(modulo, "mod_dom_local");
 mod_user_local=dlsym(modulo, "mod_user_local");
 mod_ret_alias=dlsym(modulo, "mod_ret_alias");
 mod_auth=dlsym(modulo, "mod_auth");
 mod_test_quota=dlsym(modulo, "mod_test_quota");
 mod_msg_error=dlsym(modulo, "mod_msg_error");
 mod_shutdown=dlsym(modulo, "mod_shutdown");
 
 if(!mod_init || !mod_dom_local || !mod_user_local ||
    !mod_ret_alias || !mod_auth || !mod_test_quota ||
    !mod_msg_error || !mod_shutdown){
  printf("Invalid module\n");
  return 1;
 }
 ret1=mod_init(ptr_param, mod_id, mod_version, mod_info_buffer, sizeof(mod_info_buffer));
 if(ret1){
  // Erro no modulo
  printf("Module ERROR: %s\n", mod_msg_error());
  return 1;
 }
 // Modulo carregado e ativo
 printf("Module Ready (ID:%s VERSION:%s). Type 'help' for command list\n", mod_id, mod_version);
 while((byte=fgetc(stdin)) != EOF){
  cmd[cont1++]=byte;
  if(cont1 >= TAM_cmd){
   printf("-ERROR Cmd too big\n");
   cmd[0]=0;
   cont1=0;
   continue;
  }
  if(byte == 10){
   cmd[--cont1]=0;
   if(!strncmp(cmd, "quit", 4))break;
   else if(!strncmp(cmd, "help", 4)){
    printf(" Command               |   Description\n\n");
    printf("  local <domain>            Test if the domain is a local domain\n");
    printf("  user  <mail>              Test if the mail exists\n");
    printf("  alias <mail>              Return the alias of mail\n");
    printf("  auth  <mail> <pass>       Test authentication for this account\n");
    printf("  qtest <mail>              Test if the quota of this mail is over quota\n");
    printf("\n");
   
   // -- LOCAL --
   }else if(!strncmp(cmd, "local ", 6)){
    ptr_aux1=&cmd[6];
    while(*ptr_aux1 == ' ')ptr_aux1++;
    adomain[0]=0;
    ret1=mod_dom_local(ptr_aux1, adomain);
    if(ret1 < 0)printf("ERROR: %s\n", mod_msg_error());
    else if(ret1 == 1){
     printf("OK: Local domain");
     if(adomain[0])printf(" and redirect to %s", adomain);
     printf("\n");
    }else printf("FAIL: Domain is not local\n");
    
   // -- USER --
   }else if(!strncmp(cmd, "user ", 5)){
    ptr_aux1=&cmd[5];
    while(*ptr_aux1 == ' ')ptr_aux1++;
    ret1=mod_user_local(ptr_aux1, mod_user_info, MOD_user_ncampos);
    if(ret1 < 0)printf("ERROR: %s\n", mod_msg_error());
    else if(ret1)printf("OK: User found\n");
    else printf("FAIL: User not found\n");
   
   // -- ALIAS --
   }else if(!strncmp(cmd, "alias ", 6)){
    ptr_aux1=&cmd[6];
    while(*ptr_aux1 == ' ')ptr_aux1++;
    mod_prox_bloco=0;
    do{
     ret1=mod_ret_alias(ptr_aux1, buffer_ret[0], NUM_itens_buffer, TAM_itens_buffer, &mod_prox_bloco);
     if(ret1 < 0)printf("ERROR: %s\n", mod_msg_error());
     else if(ret1)for(cont2=0; cont2<ret1; cont2++)printf("OK: %s\n", buffer_ret[cont2]);
     else printf("FAIL: User not found\n");
    }while(ret1 == NUM_itens_buffer);
   
   // -- AUTH --
   }else if(!strncmp(cmd, "auth ", 5)){
    ptr_aux1=&cmd[5];
    while(*ptr_aux1 == ' ')ptr_aux1++;
    ptr_aux2=strchr(ptr_aux1, ' ');      // Procurnado posicao da senha
    if(ptr_aux2){
     *ptr_aux2++=0;
     while(*ptr_aux2 == ' ')ptr_aux2++;
     ret1=mod_auth(ptr_aux1, ptr_aux2, mod_user_info, MOD_user_ncampos);
     if(ret1 < 0)printf("ERROR: %s\n", mod_msg_error());
     else if(!ret1)printf("FAIL: User not found\n");
     else if(ret1 == 1)printf("OK: Authentication is successful\n");
     else if(ret1 == 2)printf("FAIL: Authentication Failed\n");
    }else printf("ERROR: You need to specify a password after the username\n");
   
   // -- QTEST --
   }else if(!strncmp(cmd, "qtest ", 6)){
    ptr_aux1=&cmd[6];
    while(*ptr_aux1 == ' ')ptr_aux1++;
    ret1=mod_test_quota(ptr_aux1, mod_user_info, MOD_user_ncampos);
    if(ret1 < 0)printf("ERROR: %s\n", mod_msg_error());
    else if(!ret1)printf("FAIL: User not found\n");
    else if(ret1 == 1)printf("OK: The quota was not exceeded\n");
    else if(ret1 == 2)printf("FAIL: User is over quota\n");
   }else printf("ERROR: command not found\n");
   cmd[0]=0;
   cont1=0;
  }
 }
 mod_shutdown();
 return 0;
}







